/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifdef __SPU__

#ifndef _CBRTF4_H_
#define _CBRTF4_H_		1

#include <spu_intrinsics.h>

/*
 * FUNCTION
 *	vector float _cbrtf4(vector float x)
 *
 * DESCRIPTION
 *	The _cbrtf4 function computes the real cube roots of the elements of 
 *      the vector float x.
 *
 */
static __inline vector float _cbrtf4(vector float in)
{
  double cbrt_factors[5] = {
    0.629960524947436484311,	/* 2^(-2/3) */
    0.793700525984099680699,	/* 2^(-1/3) */
    1.0,				/* 2^(0)    */
    1.259921049894873164666,	/* 2^(1/3)  */
    1.587401051968199583441	/* 2^(2/3)  */
  };

  vec_uchar16 dw_merge = (vec_uchar16) { 0,1,2,3, 16,17,18,19, 8,9,10,11, 24,25,26,27 };
  vec_uchar16 dw_pack = (vec_uchar16) { 0,1,2,3,4,5,6,7, 16,17,18,19,20,21,22,23 };
  vec_int4 exp, bias;
  vec_uint4 mask, e_div_3, e_mod_3;
  vec_uint4 mant_mask = spu_splats((unsigned int)0x7FFFFF);
  vec_float4 half = spu_splats(0.5f);
  vec_float4 onef = spu_splats(1.0f);
  vec_float4 out, mant, ym, bf, inv_bf, bf0, bf1;
  vec_double2 two = spu_splats(2.0);
  /* Polynomial coefficients */
  vec_double2 c2 = spu_splats(0.191502161678719066);
  vec_double2 c1 = spu_splats(0.697570460207922770);
  vec_double2 c0 = spu_splats(0.492659620528969547);
  vec_double2 a0, a1, b0, b1, inv_b0, inv_b1, ym0, ym1;
  vec_double2 mant0, mant1, u0, u1, u0_3, u1_3, factor0, factor1;


  /* Normalize the mantissa (fraction part) into the range [0.5, 1.0) and 
   * extract the exponent. 
   */
  mant = spu_sel(half, in, mant_mask);
  exp = spu_and(spu_rlmask((vec_int4)in, -23), 0xFF);

  /* Generate mask used to zero result if the exponent is zero (ie, in is either
   * zero or a denorm
   */
  mask = spu_cmpeq(exp, 0);
  exp = spu_add(exp, -126);

  mant0 = spu_extend(mant);
  mant1 = spu_extend(spu_rlqwbyte(mant, 4));

  u0 = spu_madd(mant0, spu_nmsub(mant0, c2, c1), c0);
  u1 = spu_madd(mant1, spu_nmsub(mant1, c2, c1), c0);
  u0_3 = spu_mul(spu_mul(u0, u0), u0);
  u1_3 = spu_mul(spu_mul(u1, u1), u1);

  /* Compute: e_div_3 = exp/3
   *
   * Fetch:   factor = factor[2+exp%3]
   * 
   * The factors array contains 5 values: 2^(-2/3), 2^(-1/3), 2^0, 2^(1/3), 2^(2/3), 2^1.
   */
  bias = spu_rlmask(spu_rlmaska(exp, -15), -16);
  e_div_3 = (vec_uint4)spu_rlmaska(spu_madd((vec_short8)exp, spu_splats((signed short)0x5556), bias), -16);

  e_mod_3 = (vec_uint4)spu_sub((vec_int4)(exp), spu_mulo((vec_short8)e_div_3, spu_splats((signed short)3)));

  e_mod_3 = spu_add(e_mod_3, 2);

  factor0 = spu_shuffle(spu_promote(cbrt_factors[spu_extract(e_mod_3, 0)], 0),
			spu_promote(cbrt_factors[spu_extract(e_mod_3, 2)], 0),
			dw_pack);
  factor1 = spu_shuffle(spu_promote(cbrt_factors[spu_extract(e_mod_3, 1)], 0),
			spu_promote(cbrt_factors[spu_extract(e_mod_3, 3)], 0),
			dw_pack);

  /* Compute the estimated mantissa cube root (ym) equals:
   *       ym = (u * factor * (2.0 * mant + u3)) / (2.0 * u3 + mant);
   */
  a0 = spu_mul(spu_mul(factor0, u0), spu_madd(two, mant0, u0_3));
  a1 = spu_mul(spu_mul(factor1, u1), spu_madd(two, mant1, u1_3));
  b0 = spu_madd(two, u0_3, mant0);
  b1 = spu_madd(two, u1_3, mant1);

  bf0 = spu_roundtf(b0);
  bf1 = spu_roundtf(b1);

  bf = spu_shuffle(bf0, bf1, dw_merge);

  inv_bf = spu_re(bf);
  inv_bf = spu_madd(spu_nmsub(bf, inv_bf, onef), inv_bf, inv_bf);

  inv_b0 = spu_extend(inv_bf);
  inv_b1 = spu_extend(spu_rlqwbyte(inv_bf, 4));

  ym0 = spu_mul(a0, inv_b0);
  ym1 = spu_mul(a1, inv_b1);
  ym0 = spu_madd(spu_nmsub(b0, ym0, a0), inv_b0, ym0);
  ym1 = spu_madd(spu_nmsub(b1, ym1, a1), inv_b1, ym1);

  ym = spu_shuffle(spu_roundtf(ym0), spu_roundtf(ym1), dw_merge);

  /* Merge sign, computed exponent, and computed mantissa.
   */
  exp = spu_rl(spu_add((vec_int4)e_div_3, 127), 23);
  out = spu_sel((vec_float4)exp, in, spu_splats(0x80000000));
  out = spu_mul(out, ym);

  out = spu_andc(out, (vec_float4)mask);

  return (out);
}

#endif /* _CBRTF4_H_ */
#endif /* __SPU__ */
