/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifdef __SPU__
#ifndef _DIVF4_FAST_H_
#define _DIVF4_FAST_H_	1

#include <spu_intrinsics.h>

/*
 * FUNCTION
 * 	vector float _divf4_fast(vector float dividend, vector float divisor)
 * 
 * DESCRIPTION
 * 	The _divf4_fast function divides the vector dividend by the vector 
 *      divisor and returns the resulting vector quotient.
 *
 */
static __inline vector float _divf4_fast(vector float a, vector float b)
{
  vector float x0, q0, q1, q2, err;

  x0 = spu_re(b);
  q0 = spu_mul(a, x0);
  q1 = spu_nmsub(b, q0, a);
  q1 = spu_madd(x0, q1, q0);

  /* Conditionally bias the result by 1 ulp so that 
   * even the fast solution preserves the identiy function
   *  a = a / 1
   */
  q2 = (vec_float4)spu_add((vec_uint4)(q1), 1);
  err = spu_nmsub(b, q2, a);
  q2 = spu_sel(q1, q2, spu_cmpgt((vec_int4)err, -1));
  return (q2);

}

#endif /* _DIVF4_FAST_H_ */
#endif /* __SPU__ */
