/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2008,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifdef __SPU__
#ifndef _FREXPD2_H_
#define _FREXPD2_H_	1

#include <spu_intrinsics.h>



/*
 * FUNCTION
 *	vector double _frexpd2(vector double x, vector signed long long *pexp)
 *
 * DESCRIPTION
 *      The frexpd2 function returns the normalized fraction and exponent
 *      to the number x.  Double precision denorms are treated as zero 
 *      and return a 0 fraction and 0 exponent. The results are undefined 
 *      for infinities and NaNs (double precision only).
 *
 *      The two exponents are returned in elements of the vector pointed 
 *      to by pexp.
 */
static __inline vector double _frexpd2(vector double x, vector signed long long *pexp)
{
  vec_int4    exp;
  vec_uint4   naninf;
  vec_uint4   zdenorm;
  vec_double2 mant;
  vec_double2 half      = spu_splats(0.5);
  vec_ullong2 exp_mask  = spu_splats(0x7FF0000000000000ULL);
  vec_ullong2 sign_mask = spu_splats(0x8000000000000000ULL);
  vec_uchar16 dup_hi    = ((vec_uchar16) { 0,0,0,0, 0,0,0,0, 
                                           8,8,8,8, 8,8,8,8 });
  vec_uchar16 even_to_odd = ((vec_uchar16) { 0x80,0x80,0x80,0x80, 
                                             0,1,2,3, 
                                             0x80,0x80,0x80,0x80, 
                                             8,9,10,11 });

  /* Normalize the mantissa (fraction part).
   */
  mant = spu_sel(x, half, exp_mask);

  /* Zero the mantissa if the input is a denorm or zero.
   * Save the sign bit.
   */
  exp = spu_and(spu_rlmask((vec_int4)x, -20), 0x7FF);
  zdenorm = spu_cmpeq(exp, 0);
  zdenorm = spu_shuffle(zdenorm, zdenorm, dup_hi);
  mant = spu_andc(mant, (vec_double2)spu_andc((vec_ullong2)zdenorm, sign_mask));

  /* For NaNs or +- infinite, return the input value. The exponent
   * returned is unimportant in this case.
   */
  naninf = spu_cmpeq(exp, 2047);
  naninf = spu_shuffle(naninf, naninf, dup_hi);
  mant = spu_sel(mant, x, (vec_ullong2)naninf);

  /* Zero exponent if zero or denorm input. Otherwise, compute
   * exponent by removing the bias.
   */
  exp = spu_andc(spu_add(exp, ((vec_int4) { -1022, 0, -1022, 0} )), (vec_int4)zdenorm);
  exp = spu_shuffle(exp, exp, even_to_odd);

  *pexp = spu_extend(exp);

  return (mant);
}

#endif /* _FREXPD2_H_ */
#endif /* __SPU__ */
